@@ -24,7 +24,7 @@ import (
 	"fmt"
 	solrv1beta1 "github.com/apache/solr-operator/api/v1beta1"
 	"github.com/apache/solr-operator/controllers/util"
-	. "github.com/onsi/ginkgo"
+	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
@@ -75,7 +75,7 @@ var _ = FDescribe("SolrCloud controller - TLS", func() {
 
 		By("defaulting the missing SolrCloud values")
 		expectSolrCloudWithChecks(ctx, solrCloud, func(g Gomega, found *solrv1beta1.SolrCloud) {
-			g.Expect(found.WithDefaults()).To(BeFalse(), "The SolrCloud spec should not need to be defaulted eventually")
+			g.Expect(found.WithDefaults(logger)).To(BeFalse(), "The SolrCloud spec should not need to be defaulted eventually")
 		})
 	})
 
@@ -98,7 +98,10 @@ var _ = FDescribe("SolrCloud controller - TLS", func() {
 			By("checking that the User supplied TLS Config is correct in the generated StatefulSet")
 			verifyReconcileUserSuppliedTLS(ctx, solrCloud, false, false)
 			By("checking that the BasicAuth works as expected when using TLS")
-			expectStatefulSetBasicAuthConfig(ctx, solrCloud, true)
+			foundStatefulSet := expectStatefulSetBasicAuthConfig(ctx, solrCloud, true)
+
+			By("Checking that the Service has the correct settings")
+			expectTLSService(ctx, solrCloud, foundStatefulSet.Spec.Selector.MatchLabels, true)
 
 			By("Checking that the Ingress will passthrough with Server TLS")
 			expectPassthroughIngressTLSConfig(ctx, solrCloud, tlsSecretName)
@@ -122,7 +125,7 @@ var _ = FDescribe("SolrCloud controller - TLS", func() {
 			}
 			solrCloud.Spec.CustomSolrKubeOptions.PodOptions = &solrv1beta1.PodOptions{
 				ReadinessProbe: &corev1.Probe{
-					Handler: corev1.Handler{
+					ProbeHandler: corev1.ProbeHandler{
 						HTTPGet: &corev1.HTTPGetAction{
 							Scheme: corev1.URISchemeHTTPS,
 							Path:   "/solr/admin/info/health",
@@ -141,7 +144,10 @@ var _ = FDescribe("SolrCloud controller - TLS", func() {
 			By("checking that the User supplied TLS Config is correct in the generated StatefulSet")
 			verifyReconcileUserSuppliedTLS(ctx, solrCloud, false, false)
 			By("checking that the BasicAuth works as expected when using TLS")
-			expectStatefulSetBasicAuthConfig(ctx, solrCloud, true)
+			foundStatefulSet := expectStatefulSetBasicAuthConfig(ctx, solrCloud, true)
+
+			By("Checking that the Service has the correct settings")
+			expectTLSService(ctx, solrCloud, foundStatefulSet.Spec.Selector.MatchLabels, true)
 		})
 	})
 
@@ -223,10 +229,14 @@ var _ = FDescribe("SolrCloud controller - TLS", func() {
 				MountedTLSDir: mountedClientTLSDir,
 				CheckPeerName: true,
 			}
+			solrCloud.Spec.SolrAddressability.External.HideNodes = true
 		})
 		FIt("has the correct resources", func() {
 			By("checking that the Mounted TLS Config is correct in the generated StatefulSet")
-			expectStatefulSetMountedTLSDirConfig(ctx, solrCloud)
+			foundStatefulSet := expectStatefulSetMountedTLSDirConfig(ctx, solrCloud)
+
+			By("Checking that the Service has the correct settings")
+			expectTLSService(ctx, solrCloud, foundStatefulSet.Spec.Selector.MatchLabels, true)
 		})
 	})
 
@@ -372,9 +382,11 @@ var _ = FDescribe("SolrCloud controller - TLS", func() {
 		tlsSecretName := "tls-cert-secret-from-user"
 		BeforeEach(func() {
 			solrCloud.Spec.SolrSecurity = &solrv1beta1.SolrSecurityOptions{AuthenticationType: solrv1beta1.Basic}
-			solrCloud.Spec.SolrAddressability.External.IngressTLSTerminationSecret = tlsSecretName
+			solrCloud.Spec.SolrAddressability.External.IngressTLSTermination = &solrv1beta1.SolrIngressTLSTermination{
+				TLSSecret: tlsSecretName,
+			}
 		})
-		FIt("has the correct resources", func() {
+		FIt("has the correct resources - Explicit Secret", func() {
 			By("Checking that the Ingress will terminate TLS")
 			expectTerminateIngressTLSConfig(ctx, solrCloud, tlsSecretName, false)
 
@@ -384,6 +396,36 @@ var _ = FDescribe("SolrCloud controller - TLS", func() {
 				g.Expect(found.ExternalCommonAddress).To(Not(BeNil()), "External common address in Status should not be nil.")
 				g.Expect(*found.ExternalCommonAddress).To(Equal("https://"+solrCloud.Namespace+"-"+solrCloud.Name+"-solrcloud."+testDomain), "Wrong external common address in status")
 			})
+
+			foundStatefulSet := expectStatefulSet(ctx, solrCloud, solrCloud.StatefulSetName())
+
+			By("Checking that the Service has the correct settings")
+			expectTLSService(ctx, solrCloud, foundStatefulSet.Spec.Selector.MatchLabels, false)
+		})
+	})
+
+	FContext("Common Ingress TLS Termination - Default Secret", func() {
+		BeforeEach(func() {
+			solrCloud.Spec.SolrSecurity = &solrv1beta1.SolrSecurityOptions{AuthenticationType: solrv1beta1.Basic}
+			solrCloud.Spec.SolrAddressability.External.IngressTLSTermination = &solrv1beta1.SolrIngressTLSTermination{
+				UseDefaultTLSSecret: true,
+			}
+		})
+		FIt("has the correct resources", func() {
+			By("Checking that the Ingress will terminate TLS")
+			expectTerminateIngressTLSConfig(ctx, solrCloud, "", false)
+
+			By("Checking that the SolrCloud status has the correct scheme for URLs")
+			expectSolrCloudStatusWithChecks(ctx, solrCloud, func(g Gomega, found *solrv1beta1.SolrCloudStatus) {
+				g.Expect(found.InternalCommonAddress).To(Equal("http://"+solrCloud.Name+"-solrcloud-common."+solrCloud.Namespace), "Wrong internal common address in status")
+				g.Expect(found.ExternalCommonAddress).To(Not(BeNil()), "External common address in Status should not be nil.")
+				g.Expect(*found.ExternalCommonAddress).To(Equal("https://"+solrCloud.Namespace+"-"+solrCloud.Name+"-solrcloud."+testDomain), "Wrong external common address in status")
+			})
+
+			foundStatefulSet := expectStatefulSet(ctx, solrCloud, solrCloud.StatefulSetName())
+
+			By("Checking that the Service has the correct settings")
+			expectTLSService(ctx, solrCloud, foundStatefulSet.Spec.Selector.MatchLabels, false)
 		})
 	})
 })
@@ -692,7 +734,7 @@ func createMockTLSSecret(ctx context.Context, parentObject client.Object, secret
 	return mockTLSSecret
 }
 
-func expectStatefulSetMountedTLSDirConfig(ctx context.Context, solrCloud *solrv1beta1.SolrCloud) {
+func expectStatefulSetMountedTLSDirConfig(ctx context.Context, solrCloud *solrv1beta1.SolrCloud) *appsv1.StatefulSet {
 	statefulSet := expectStatefulSet(ctx, solrCloud, solrCloud.StatefulSetName())
 	podTemplate := &statefulSet.Spec.Template
 	expectMountedTLSDirConfigOnPodTemplate(podTemplate, solrCloud)
@@ -718,6 +760,7 @@ func expectStatefulSetMountedTLSDirConfig(ctx context.Context, solrCloud *solrv1
 		Expect(expInitContainer.Command[2]).To(Not(ContainSubstring("SOLR_SSL_CLIENT_KEY_STORE_PASSWORD")), "Wrong shell command for init container: %s", name)
 		Expect(expInitContainer.Command[2]).To(Not(ContainSubstring("SOLR_SSL_CLIENT_TRUST_STORE_PASSWORD")), "Wrong shell command for init container: %s", name)
 	}
+	return statefulSet
 }
 
 func expectMountedTLSDirConfigOnPodTemplate(podTemplate *corev1.PodTemplateSpec, solrCloud *solrv1beta1.SolrCloud) {
@@ -816,6 +859,47 @@ func expectZkSetupInitContainerForTLSWithGomega(g Gomega, solrCloud *solrv1beta1
 	}
 }
 
+func expectTLSService(ctx context.Context, solrCloud *solrv1beta1.SolrCloud, selectorLables map[string]string, podsHaveTLSEnabled bool) {
+	appProtocol := "http"
+	podPort := 8983
+	servicePort := 80
+	if podsHaveTLSEnabled {
+		appProtocol = "https"
+		servicePort = 443
+	}
+	By("testing the Solr Common Service")
+	commonService := expectService(ctx, solrCloud, solrCloud.CommonServiceName(), selectorLables, false)
+	Expect(commonService.Spec.Ports[0].Name).To(Equal("solr-client"), "Wrong port name on common Service")
+	Expect(commonService.Spec.Ports[0].Port).To(Equal(int32(servicePort)), "Wrong port on common Service")
+	Expect(commonService.Spec.Ports[0].TargetPort.StrVal).To(Equal("solr-client"), "Wrong podPort name on common Service")
+	Expect(commonService.Spec.Ports[0].Protocol).To(Equal(corev1.ProtocolTCP), "Wrong protocol on common Service")
+	Expect(commonService.Spec.Ports[0].AppProtocol).ToNot(BeNil(), "AppProtocol on common Service should not be nil")
+	Expect(*commonService.Spec.Ports[0].AppProtocol).To(Equal(appProtocol), "Wrong appProtocol on common Service")
+
+	if solrCloud.Spec.SolrAddressability.External.UsesIndividualNodeServices() {
+		nodeNames := solrCloud.GetAllSolrPodNames()
+		By("testing the Solr Node Service(s)")
+		for _, nodeName := range nodeNames {
+			service := expectService(ctx, solrCloud, nodeName, util.MergeLabelsOrAnnotations(selectorLables, map[string]string{"statefulset.kubernetes.io/pod-name": nodeName}), false)
+			Expect(service.Spec.Ports[0].Name).To(Equal("solr-client"), "Wrong port name on common Service")
+			Expect(service.Spec.Ports[0].Port).To(Equal(int32(servicePort)), "Wrong port on node Service")
+			Expect(service.Spec.Ports[0].TargetPort.StrVal).To(Equal("solr-client"), "Wrong podPort name on node Service")
+			Expect(service.Spec.Ports[0].Protocol).To(Equal(corev1.ProtocolTCP), "Wrong protocol on node Service")
+			Expect(service.Spec.Ports[0].AppProtocol).ToNot(BeNil(), "AppProtocol on node Service should not be nil")
+			Expect(*service.Spec.Ports[0].AppProtocol).To(Equal(appProtocol), "Wrong appProtocol on node Service")
+		}
+	} else {
+		By("testing the Solr Headless Service")
+		headlessService := expectService(ctx, solrCloud, solrCloud.HeadlessServiceName(), selectorLables, true)
+		Expect(headlessService.Spec.Ports[0].Name).To(Equal("solr-client"), "Wrong port name on common Service")
+		Expect(headlessService.Spec.Ports[0].Port).To(Equal(int32(podPort)), "Wrong port on headless Service")
+		Expect(headlessService.Spec.Ports[0].TargetPort.StrVal).To(Equal("solr-client"), "Wrong podPort name on headless Service")
+		Expect(headlessService.Spec.Ports[0].Protocol).To(Equal(corev1.ProtocolTCP), "Wrong protocol on headless Service")
+		Expect(headlessService.Spec.Ports[0].AppProtocol).ToNot(BeNil(), "AppProtocol on headless Service should not be nil")
+		Expect(*headlessService.Spec.Ports[0].AppProtocol).To(Equal(appProtocol), "Wrong appProtocol on headless Service")
+	}
+}
+
 func expectPassthroughIngressTLSConfig(ctx context.Context, solrCloud *solrv1beta1.SolrCloud, expectedTLSSecretName string) {
 	ingress := expectIngress(ctx, solrCloud, solrCloud.CommonIngressName())
 	Expect(ingress.Spec.TLS).To(Not(BeEmpty()), "Ingress does not have TLS secrets provided")
