@@ -19,11 +19,13 @@ package controllers
 
 import (
 	"context"
+	"strconv"
+
 	solrv1beta1 "github.com/apache/solr-operator/api/v1beta1"
 	"github.com/apache/solr-operator/controllers/util"
-	zk_crd "github.com/apache/solr-operator/controllers/zk_api"
-	. "github.com/onsi/ginkgo"
+	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
+	zkApi "github.com/pravega/zookeeper-operator/api/v1beta1"
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
@@ -58,7 +60,7 @@ var _ = FDescribe("SolrCloud controller - Zookeeper", func() {
 
 		By("defaulting the missing SolrCloud values")
 		expectSolrCloudWithChecks(ctx, solrCloud, func(g Gomega, found *solrv1beta1.SolrCloud) {
-			g.Expect(found.WithDefaults()).To(BeFalse(), "The SolrCloud spec should not need to be defaulted eventually")
+			g.Expect(found.WithDefaults(logger)).To(BeFalse(), "The SolrCloud spec should not need to be defaulted eventually")
 		})
 	})
 
@@ -112,12 +114,12 @@ var _ = FDescribe("SolrCloud controller - Zookeeper", func() {
 				"SOLR_NODE_PORT": "8983",
 				"SOLR_OPTS":      "-DhostPort=$(SOLR_NODE_PORT) $(SOLR_ZK_CREDS_AND_ACLS) -Dextra -Dopts",
 			}
+			insertExpectedAclEnvVars(expectedEnvVars, false)
+			for _, envVar := range extraVars {
+				expectedEnvVars[envVar.Name] = envVar.Value
+			}
 			foundEnv := statefulSet.Spec.Template.Spec.Containers[0].Env
-			testACLEnvVars(foundEnv[len(foundEnv)-6:len(foundEnv)-3], false)
-			Expect(foundEnv[len(foundEnv)-3:len(foundEnv)-1]).To(Equal(extraVars), "Extra Env Vars are not the same as the ones provided in podOptions")
-			// Note that this check changes the variable foundEnv, so the values are no longer valid afterwards.
-			// TODO: Make this not invalidate foundEnv
-			testPodEnvVariables(expectedEnvVars, append(foundEnv[:len(foundEnv)-6], foundEnv[len(foundEnv)-1]))
+			testPodEnvVariables(expectedEnvVars, foundEnv)
 			Expect(statefulSet.Spec.Template.Spec.Containers[0].Lifecycle.PreStop.Exec.Command).To(Equal([]string{"solr", "stop", "-p", "8983"}), "Incorrect pre-stop command")
 		})
 	})
@@ -177,12 +179,12 @@ var _ = FDescribe("SolrCloud controller - Zookeeper", func() {
 				"SOLR_NODE_PORT": "8983",
 				"SOLR_OPTS":      "-DhostPort=$(SOLR_NODE_PORT) $(SOLR_ZK_CREDS_AND_ACLS) -Dextra -Dopts",
 			}
+			insertExpectedAclEnvVars(expectedEnvVars, true)
+			for _, envVar := range extraVars {
+				expectedEnvVars[envVar.Name] = envVar.Value
+			}
 			foundEnv := statefulSet.Spec.Template.Spec.Containers[0].Env
-			testACLEnvVars(foundEnv[len(foundEnv)-8:len(foundEnv)-3], true)
-			Expect(foundEnv[len(foundEnv)-3:len(foundEnv)-1]).To(Equal(extraVars), "Extra Env Vars are not the same as the ones provided in podOptions")
-			// Note that this check changes the variable foundEnv, so the values are no longer valid afterwards.
-			// TODO: Make this not invalidate foundEnv
-			testPodEnvVariables(expectedEnvVars, append(foundEnv[:len(foundEnv)-8], foundEnv[len(foundEnv)-1]))
+			testPodEnvVariables(expectedEnvVars, foundEnv)
 			Expect(statefulSet.Spec.Template.Spec.Containers[0].Lifecycle.PreStop.Exec.Command).To(Equal([]string{"solr", "stop", "-p", "8983"}), "Incorrect pre-stop command")
 
 			By("testing that no ZookeeperCluster is created when using a connection String")
@@ -214,9 +216,20 @@ var _ = FDescribe("SolrCloud controller - Zookeeper", func() {
 							SecurityContext:               &testPodSecurityContext,
 							TerminationGracePeriodSeconds: testTerminationGracePeriodSeconds,
 							ImagePullSecrets:              testAdditionalImagePullSecrets,
+							TopologySpreadConstraints:     testTopologySpreadConstraints,
 						},
-						Config: zkConf,
-						ChRoot: "a-ch/root",
+						Config:                 zkConf,
+						ChRoot:                 "a-ch/root",
+						Probes:                 &testZkProbes,
+						InitContainers:         extraContainers1,
+						Containers:             extraContainers2,
+						Labels:                 testConfigMapLabels,
+						MaxUnavailableReplicas: 5,
+						AdminServerService:     testZKAdminServerServicePolicy,
+						ClientService:          testZKClientServicePolicy,
+						HeadlessService:        testZKHeadlessServicePolicy,
+						Volumes:                testZkVolumes,
+						VolumeMounts:           testZkVolumeMounts,
 					},
 				},
 				CustomSolrKubeOptions: solrv1beta1.CustomSolrKubeOptions{
@@ -272,6 +285,15 @@ var _ = FDescribe("SolrCloud controller - Zookeeper", func() {
 			Expect(zkCluster.Spec.Ephemeral).To(Not(BeNil()), "ZkCluster.spec.ephemeral should not be nil")
 			Expect(zkCluster.Spec.Ephemeral.EmptyDirVolumeSource.Medium).To(BeEquivalentTo("Memory"), "Incorrect EmptyDir medium for ZK Cluster ephemeral storage")
 			Expect(zkCluster.Spec.Persistence).To(BeNil(), "ZkCluster.spec.persistence should be nil when using ephermeral storage")
+			Expect(zkCluster.Spec.Probes).To(Equal(&testZkProbes), "Incorrect zkCluster probes")
+			Expect(zkCluster.Spec.InitContainers).To(Equal(extraContainers1), "Incorrect zkCluster initContainers")
+			Expect(zkCluster.Spec.Containers).To(Equal(extraContainers2), "Incorrect zkCluster containers")
+			Expect(zkCluster.Spec.MaxUnavailableReplicas).To(BeEquivalentTo(5), "Incorrect zkCluster maxUnavailableReplicas")
+			Expect(zkCluster.Spec.AdminServerService).To(Equal(testZKAdminServerServicePolicy), "Incorrect zkCluster adminServerService")
+			Expect(zkCluster.Spec.ClientService).To(Equal(testZKClientServicePolicy), "Incorrect zkCluster clientService")
+			Expect(zkCluster.Spec.HeadlessService).To(Equal(testZKHeadlessServicePolicy), "Incorrect zkCluster headlessService")
+			Expect(zkCluster.Spec.Volumes).To(Equal(testZkVolumes), "Incorrect zkCluster volumes")
+			Expect(zkCluster.Spec.VolumeMounts).To(Equal(testZkVolumeMounts), "Incorrect zkCluster volumeMounts")
 
 			// Check ZK Pod Options
 			Expect(zkCluster.Spec.Pod.Affinity).To(Equal(testAffinity), "Incorrect zkCluster affinity")
@@ -280,11 +302,19 @@ var _ = FDescribe("SolrCloud controller - Zookeeper", func() {
 			Expect(zkCluster.Spec.Pod.Resources).To(Equal(testResources), "Incorrect zkCluster resources")
 			Expect(zkCluster.Spec.Pod.Env).To(Equal(extraVars), "Incorrect zkCluster env vars")
 			Expect(zkCluster.Spec.Pod.ServiceAccountName).To(Equal(testServiceAccountName), "Incorrect zkCluster serviceAccountName")
-			Expect(zkCluster.Spec.Pod.Labels).To(Equal(util.MergeLabelsOrAnnotations(testSSLabels, map[string]string{"app": "foo-solrcloud-zookeeper", "release": "foo-solrcloud-zookeeper"})), "Incorrect zkCluster pod labels")
 			Expect(zkCluster.Spec.Pod.Annotations).To(Equal(testSSAnnotations), "Incorrect zkCluster pod annotations")
 			Expect(zkCluster.Spec.Pod.SecurityContext).To(Equal(&testPodSecurityContext), "Incorrect zkCluster pod securityContext")
 			Expect(zkCluster.Spec.Pod.TerminationGracePeriodSeconds).To(Equal(testTerminationGracePeriodSeconds), "Incorrect zkCluster pod terminationGracePeriodSeconds")
 			Expect(zkCluster.Spec.Pod.ImagePullSecrets).To(Equal(append(append(make([]corev1.LocalObjectReference, 0), testAdditionalImagePullSecrets...), corev1.LocalObjectReference{Name: testImagePullSecretName})), "Incorrect zkCluster imagePullSecrets")
+			Expect(zkCluster.Spec.Pod.TopologySpreadConstraints).To(Equal(testTopologySpreadConstraints), "Incorrect zkCluster pod topologySpreadConstraints")
+
+			testZkPodLabels := util.MergeLabelsOrAnnotations(testSSLabels, map[string]string{"app": "foo-solrcloud-zookeeper", "release": "foo-solrcloud-zookeeper"})
+			Expect(zkCluster.Spec.Pod.Labels).To(Equal(testZkPodLabels), "Wrong zkCluster pod labels")
+			testZkClusterLabels := map[string]string{"solr-cloud": "foo", "technology": "zookeeper"}
+			Expect(zkCluster.Labels).To(Equal(testZkClusterLabels), "Wrong zkCluster labels")
+			testZkSpecLabels := util.MergeLabelsOrAnnotations(testConfigMapLabels, testZkClusterLabels)
+			testZkSpecLabels = util.MergeLabelsOrAnnotations(testZkSpecLabels, map[string]string{"app": "foo-solrcloud-zookeeper", "release": "foo-solrcloud-zookeeper"})
+			Expect(zkCluster.Spec.Labels).To(Equal(testZkSpecLabels), "Wrong zkCluster spec labels")
 
 			// Check ZK Config Options
 			Expect(zkCluster.Spec.Conf.InitLimit).To(Equal(zkConf.InitLimit), "Incorrect zkCluster Config InitLimit")
@@ -430,12 +460,12 @@ var _ = FDescribe("SolrCloud controller - Zookeeper", func() {
 				"ZK_CHROOT":      "/a-ch/root",
 				"SOLR_OPTS":      "-DhostPort=$(SOLR_NODE_PORT) $(SOLR_ZK_CREDS_AND_ACLS) -Dextra -Dopts",
 			}
+			insertExpectedAclEnvVars(expectedEnvVars, false)
+			for _, envVar := range extraVars {
+				expectedEnvVars[envVar.Name] = envVar.Value
+			}
 			foundEnv := statefulSet.Spec.Template.Spec.Containers[0].Env
-			testACLEnvVars(foundEnv[len(foundEnv)-6:len(foundEnv)-3], false)
-			Expect(foundEnv[len(foundEnv)-3:len(foundEnv)-1]).To(Equal(extraVars), "Extra Env Vars are not the same as the ones provided in podOptions")
-			// Note that this check changes the variable foundEnv, so the values are no longer valid afterwards.
-			// TODO: Make this not invalidate foundEnv
-			testPodEnvVariables(expectedEnvVars, append(foundEnv[:len(foundEnv)-6], foundEnv[len(foundEnv)-1]))
+			testPodEnvVariables(expectedEnvVars, foundEnv)
 			Expect(statefulSet.Spec.Template.Spec.Containers[0].Lifecycle.PreStop.Exec.Command).To(Equal([]string{"solr", "stop", "-p", "8983"}), "Incorrect pre-stop command")
 
 			By("testing the created ZookeeperCluster")
@@ -505,26 +535,66 @@ var _ = FDescribe("SolrCloud controller - Zookeeper", func() {
 				"SOLR_NODE_PORT": "8983",
 				"SOLR_OPTS":      "-DhostPort=$(SOLR_NODE_PORT) $(SOLR_ZK_CREDS_AND_ACLS) -Dextra -Dopts",
 			}
+			insertExpectedAclEnvVars(expectedEnvVars, true)
+			for _, envVar := range extraVars {
+				expectedEnvVars[envVar.Name] = envVar.Value
+			}
 			foundEnv := statefulSet.Spec.Template.Spec.Containers[0].Env
-			testACLEnvVars(foundEnv[len(foundEnv)-8:len(foundEnv)-3], true)
-			Expect(foundEnv[len(foundEnv)-3:len(foundEnv)-1]).To(Equal(extraVars), "Extra Env Vars are not the same as the ones provided in podOptions")
-			// Note that this check changes the variable foundEnv, so the values are no longer valid afterwards.
-			// TODO: Make this not invalidate foundEnv
-			testPodEnvVariables(expectedEnvVars, append(foundEnv[:len(foundEnv)-8], foundEnv[len(foundEnv)-1]))
+			testPodEnvVariables(expectedEnvVars, foundEnv)
 			Expect(statefulSet.Spec.Template.Spec.Containers[0].Lifecycle.PreStop.Exec.Command).To(Equal([]string{"solr", "stop", "-p", "8983"}), "Incorrect pre-stop command")
 
 			By("testing the created ZookeeperCluster")
 			expectZookeeperCluster(ctx, solrCloud, solrCloud.ProvidedZookeeperName())
 		})
 	})
+
+	FContext("Solr Cloud with Solr ZK Connection Options", func() {
+		BeforeEach(func() {
+			solrCloud.Spec = solrv1beta1.SolrCloudSpec{
+				ZookeeperRef: &solrv1beta1.ZookeeperRef{
+					ConnectionInfo: &solrv1beta1.ZookeeperConnectionInfo{
+						InternalConnectionString: "host:7271",
+						ChRoot:                   "/test",
+					},
+				},
+				SolrZkOpts: testSolrZKOpts,
+				SolrOpts:   testSolrOpts,
+				SolrSecurity: &solrv1beta1.SolrSecurityOptions{
+					AuthenticationType: solrv1beta1.Basic,
+				},
+			}
+		})
+		FIt("has the correct resources", func() {
+			By("testing the Solr StatefulSet")
+			statefulSet := expectStatefulSet(ctx, solrCloud, solrCloud.StatefulSetName())
+			Expect(statefulSet.Spec.Template.Spec.Containers).To(HaveLen(1), "Solr StatefulSet requires a container.")
+			expectedEnvVars := map[string]string{
+				"ZK_HOST":        "host:7271/test",
+				"SOLR_HOST":      "$(POD_HOSTNAME).foo-solrcloud-headless.default",
+				"SOLR_PORT":      "8983",
+				"SOLR_NODE_PORT": "8983",
+				"SOLR_ZK_OPTS":   testSolrZKOpts,
+				"SOLR_OPTS":      "-DhostPort=$(SOLR_NODE_PORT) $(SOLR_ZK_OPTS) " + testSolrOpts,
+				"SOLR_STOP_WAIT": strconv.FormatInt(60-5, 10),
+			}
+			testPodEnvVariables(expectedEnvVars, statefulSet.Spec.Template.Spec.Containers[0].Env)
+
+			expectedInitContainerEnvVars := map[string]string{
+				"SOLR_ZK_OPTS":    testSolrZKOpts,
+				"SOLR_OPTS":       "$(SOLR_ZK_OPTS) " + testSolrOpts,
+				"ZKCLI_JVM_FLAGS": "-Dsolr.zk.opts=this",
+			}
+			testPodEnvVariables(expectedInitContainerEnvVars, statefulSet.Spec.Template.Spec.InitContainers[1].Env)
+		})
+	})
 })
 
-func expectZookeeperCluster(ctx context.Context, parentResource client.Object, zkName string, additionalOffset ...int) *zk_crd.ZookeeperCluster {
+func expectZookeeperCluster(ctx context.Context, parentResource client.Object, zkName string, additionalOffset ...int) *zkApi.ZookeeperCluster {
 	return expectZookeeperClusterWithChecks(ctx, parentResource, zkName, nil, resolveOffset(additionalOffset))
 }
 
-func expectZookeeperClusterWithChecks(ctx context.Context, parentResource client.Object, zkName string, additionalChecks func(Gomega, *zk_crd.ZookeeperCluster), additionalOffset ...int) *zk_crd.ZookeeperCluster {
-	found := &zk_crd.ZookeeperCluster{}
+func expectZookeeperClusterWithChecks(ctx context.Context, parentResource client.Object, zkName string, additionalChecks func(Gomega, *zkApi.ZookeeperCluster), additionalOffset ...int) *zkApi.ZookeeperCluster {
+	found := &zkApi.ZookeeperCluster{}
 	EventuallyWithOffset(resolveOffset(additionalOffset), func(g Gomega) {
 		g.Expect(k8sClient.Get(ctx, resourceKey(parentResource, zkName), found)).To(Succeed(), "Expected ZookeeperCluster does not exist")
 		if additionalChecks != nil {
@@ -536,7 +606,7 @@ func expectZookeeperClusterWithChecks(ctx context.Context, parentResource client
 	ExpectWithOffset(resolveOffset(additionalOffset), k8sClient.Delete(ctx, found)).To(Succeed())
 	Eventually(
 		func(g Gomega) types.UID {
-			newResource := &zk_crd.ZookeeperCluster{}
+			newResource := &zkApi.ZookeeperCluster{}
 			g.Expect(k8sClient.Get(ctx, resourceKey(parentResource, zkName), newResource)).To(Succeed(), "ZookeeperCluster not recreated after deletion")
 			return newResource.UID
 		}).Should(And(Not(BeEmpty()), Not(Equal(found.UID))), "New ZookeeperCluster, with new UID, not created.")
@@ -545,6 +615,6 @@ func expectZookeeperClusterWithChecks(ctx context.Context, parentResource client
 
 func expectNoZookeeperCluster(ctx context.Context, parentResource client.Object, zkName string, additionalOffset ...int) {
 	ConsistentlyWithOffset(resolveOffset(additionalOffset), func() error {
-		return k8sClient.Get(ctx, resourceKey(parentResource, zkName), &zk_crd.ZookeeperCluster{})
+		return k8sClient.Get(ctx, resourceKey(parentResource, zkName), &zkApi.ZookeeperCluster{})
 	}).Should(MatchError("zookeeperclusters.zookeeper.pravega.io \""+zkName+"\" not found"), "ZookeeperCluster exists when it should not")
 }
