@@ -34,6 +34,7 @@ const (
 	SolrMetricsPortName = "solr-metrics"
 	ExtSolrMetricsPort  = 80
 
+	SolrPrometheusExporterContainer          = "solr-prometheus-exporter"
 	DefaultPrometheusExporterEntrypoint      = "/opt/solr/contrib/prometheus-exporter/bin/solr-exporter"
 	PrometheusExporterConfigMapKey           = "solr-prometheus-exporter.xml"
 	PrometheusExporterConfigXmlMd5Annotation = "solr.apache.org/exporterConfigXmlMd5"
@@ -48,7 +49,7 @@ type SolrConnectionInfo struct {
 
 // GenerateSolrPrometheusExporterDeployment returns a new appsv1.Deployment pointer generated for the SolrCloud Prometheus Exporter instance
 // solrPrometheusExporter: SolrPrometheusExporter instance
-func GenerateSolrPrometheusExporterDeployment(solrPrometheusExporter *solr.SolrPrometheusExporter, solrConnectionInfo SolrConnectionInfo, configXmlMd5 string, tls *TLSCerts, basicAuthMd5 string) *appsv1.Deployment {
+func GenerateSolrPrometheusExporterDeployment(solrPrometheusExporter *solr.SolrPrometheusExporter, solrConnectionInfo SolrConnectionInfo, solrCloudImage *solr.ContainerImage, configXmlMd5 string, tls *TLSCerts, basicAuthMd5 string) *appsv1.Deployment {
 	gracePeriodTerm := int64(10)
 	singleReplica := int32(1)
 	fsGroup := int64(SolrMetricsPort)
@@ -175,30 +176,51 @@ func GenerateSolrPrometheusExporterDeployment(solrPrometheusExporter *solr.SolrP
 		envVars = append(envVars, corev1.EnvVar{Name: "JAVA_OPTS", Value: strings.Join(allJavaOpts, " ")})
 	}
 
+	containerImage := solrPrometheusExporter.Spec.Image
+	if containerImage == nil {
+		containerImage = solrCloudImage
+	}
+
+	defaultProbeHandler := corev1.ProbeHandler{
+		HTTPGet: &corev1.HTTPGetAction{
+			Scheme: corev1.URISchemeHTTP,
+			Path:   "/metrics",
+			Port:   intstr.FromInt(SolrMetricsPort),
+		},
+	}
+
 	containers := []corev1.Container{
 		{
-			Name:            "solr-prometheus-exporter",
-			Image:           solrPrometheusExporter.Spec.Image.ToImageName(),
-			ImagePullPolicy: solrPrometheusExporter.Spec.Image.PullPolicy,
+			Name:            SolrPrometheusExporterContainer,
+			Image:           containerImage.ToImageName(),
+			ImagePullPolicy: containerImage.PullPolicy,
 			Ports:           []corev1.ContainerPort{{ContainerPort: SolrMetricsPort, Name: SolrMetricsPortName, Protocol: corev1.ProtocolTCP}},
 			VolumeMounts:    volumeMounts,
 			Command:         []string{entrypoint},
 			Args:            exporterArgs,
 			Env:             envVars,
 
-			LivenessProbe: &corev1.Probe{
-				Handler: corev1.Handler{
-					HTTPGet: &corev1.HTTPGetAction{
-						Scheme: corev1.URISchemeHTTP,
-						Path:   "/metrics",
-						Port:   intstr.FromInt(SolrMetricsPort),
-					},
-				},
-				InitialDelaySeconds: 20,
-				TimeoutSeconds:      1,
-				PeriodSeconds:       10,
+			StartupProbe: &corev1.Probe{
+				ProbeHandler:        defaultProbeHandler,
+				InitialDelaySeconds: 2,
+				TimeoutSeconds:      2,
+				PeriodSeconds:       2,
 				SuccessThreshold:    1,
-				FailureThreshold:    3,
+				FailureThreshold:    10,
+			},
+			LivenessProbe: &corev1.Probe{
+				ProbeHandler:     defaultProbeHandler,
+				TimeoutSeconds:   2,
+				PeriodSeconds:    10,
+				SuccessThreshold: 1,
+				FailureThreshold: 3,
+			},
+			ReadinessProbe: &corev1.Probe{
+				ProbeHandler:     defaultProbeHandler,
+				TimeoutSeconds:   2,
+				PeriodSeconds:    5,
+				SuccessThreshold: 1,
+				FailureThreshold: 3,
 			},
 		},
 	}
@@ -261,10 +283,10 @@ func GenerateSolrPrometheusExporterDeployment(solrPrometheusExporter *solr.SolrP
 		},
 	}
 
-	if solrPrometheusExporter.Spec.Image.ImagePullSecret != "" {
+	if containerImage.ImagePullSecret != "" {
 		imagePullSecrets = append(
 			imagePullSecrets,
-			corev1.LocalObjectReference{Name: solrPrometheusExporter.Spec.Image.ImagePullSecret},
+			corev1.LocalObjectReference{Name: containerImage.ImagePullSecret},
 		)
 	}
 
@@ -384,7 +406,6 @@ func GenerateSolrMetricsService(solrPrometheusExporter *solr.SolrPrometheusExpor
 		"prometheus.io/scrape": "true",
 		"prometheus.io/scheme": "http",
 		"prometheus.io/path":   "/metrics",
-		"prometheus.io/port":   strconv.Itoa(ExtSolrMetricsPort),
 	}
 
 	selectorLabels := solrPrometheusExporter.SharedLabels()
@@ -396,6 +417,8 @@ func GenerateSolrMetricsService(solrPrometheusExporter *solr.SolrPrometheusExpor
 		annotations = MergeLabelsOrAnnotations(annotations, customOptions.Annotations)
 	}
 
+	appProtocol := "http"
+
 	service := &corev1.Service{
 		ObjectMeta: metav1.ObjectMeta{
 			Name:        solrPrometheusExporter.MetricsServiceName(),
@@ -405,7 +428,13 @@ func GenerateSolrMetricsService(solrPrometheusExporter *solr.SolrPrometheusExpor
 		},
 		Spec: corev1.ServiceSpec{
 			Ports: []corev1.ServicePort{
-				{Name: SolrMetricsPortName, Port: ExtSolrMetricsPort, Protocol: corev1.ProtocolTCP, TargetPort: intstr.FromInt(SolrMetricsPort)},
+				{
+					Name:        SolrMetricsPortName,
+					Port:        ExtSolrMetricsPort,
+					Protocol:    corev1.ProtocolTCP,
+					TargetPort:  intstr.FromInt(SolrMetricsPort),
+					AppProtocol: &appProtocol,
+				},
 			},
 			Selector: selectorLabels,
 		},
